<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class RPAEFW_Shipping_Method extends WC_Shipping_Method {
	public function __construct( $instance_id = 0 ) {
		$this->id                 = 'rpaefw_post_calc';
		$this->instance_id        = absint( $instance_id );
		$this->method_title       = esc_html__( 'Russian Post', 'russian-post-and-ems-for-woocommerce' );
		$this->method_description = esc_html__( 'The plugin allows you to automatically calculate shipping costs "Russian Post" or "EMS" on the checkout page using postcalc.ru service.', 'russian-post-and-ems-for-woocommerce' );
		$this->supports           = array(
			'shipping-zones',
			'instance-settings'
		);
		$this->instance_form_fields = array(
			'title'   => array(
				'title'   			=> esc_html__( 'Title', 'russian-post-and-ems-for-woocommerce' ),
				'type'    			=> 'text',
				'default' 			=> esc_html__( 'Russian Post', 'russian-post-and-ems-for-woocommerce' ),
				'custom_attributes' => array(
					'required' => 'required'
				),
			),
			'from'    => array(
				'title'       		=> esc_html__( 'Оrigin Postcode', 'russian-post-and-ems-for-woocommerce' ),
				'description' 		=> esc_html__( '6-digit code of the sender.', 'russian-post-and-ems-for-woocommerce' ),
				'type'        		=> 'number',
				'custom_attributes' => array(
					'required' => 'required'
				),
			),
			'addcost' => array(
				'title'       => esc_html__( 'Сost', 'russian-post-and-ems-for-woocommerce' ),
				'description' => esc_html__( 'Additional flat rate for shipping method. This may be the average value of the package or the cost of fuel, spent on the road to the post;)', 'russian-post-and-ems-for-woocommerce' ),
				'type'        => 'number',
				'default'     => 0,
			),
			'type'    => array(
				'title'   => esc_html__( 'Type', 'russian-post-and-ems-for-woocommerce' ),
				'type'    => 'select',
				'default' => 'ЦеннаяПосылка',
				'options' => array(
					'ПростаяБандероль' => 'Простая бандероль',

					'ЗаказнаяБандероль'       => 'Заказная бандероль',
					'ЗаказнаяБандероль1Класс' => 'Заказная бандероль 1 класс',

					'ЦеннаяБандероль'       => 'Ценная бандероль',
					'ЦеннаяБандероль1Класс' => 'Ценная бандероль 1 класс',

					'ПростаяПосылка' => 'Простая посылка',
					'ЦеннаяПосылка'  => 'Ценная посылка',
					'Посылка1Класс'  => 'Посылка 1 класса',

					'ЦеннаяБандероль1Класс' => 'Ценная бандероль 1 класс',
					'EMS'                   => 'Курьерская доставка EMS',

					'МждМешокМ'                  => 'Международный мешок М',
					'МждМешокМАвиа'              => 'Международный мешок М авиа',
					'МждМешокМЗаказной'          => 'Международный мешок М заказной',
					'МждМешокМАвиаЗаказной'      => 'Международный мешок М авиа заказной',
					'МждБандероль'               => 'Международная бандероль',
					'МждБандерольАвиа'           => 'Международная авиабандероль',
					'МждБандерольЗаказная'       => 'Международная бандероль заказная',
					'МждБандерольАвиаЗаказная'   => 'Международная авиабандероль заказная',
					'МждМелкийПакет'             => 'Международный мелкий пакет',
					'МждМелкийПакетАвиа'         => 'Международный мелкий пакет авиа',
					'МждМелкийПакетЗаказной'     => 'Международный мелкий пакет заказной',
					'МждМелкийПакетАвиаЗаказной' => 'Международный мелкий пакет авиа заказной',
					'МждПосылка'                 => 'Международная посылка',
					'МждПосылкаАвиа'             => 'Международная авиапосылка',
					'EMS_МждДокументы'           => 'ЕMS международное - документы',
					'EMS_МждТовары'              => 'ЕMS международное - товары',
				)
			),

			'fixedpackvalue' => array(
				'title'       => esc_html__( 'Max. Fixed Package Value', 'russian-post-and-ems-for-woocommerce' ),
				'description' => esc_html__( 'You can set max. fixed value for some types of departure. By default value equals sum of the order. You can set it to zero to make your shipments are considered without estimating the valuation.', 'russian-post-and-ems-for-woocommerce' ),
				'type'        => 'number',
			),

			// packaging
			'addpackweight'  => array(
				'title'       => esc_html__( 'Packaging', 'russian-post-and-ems-for-woocommerce' ),
				'description' => esc_html__( 'Weight of the one packaging in grams. This weight will be added to the total weight of the order.', 'russian-post-and-ems-for-woocommerce' ),
				'type'        => 'number',
				'default'     => 0,
			),
			'addpackcost'    => array(
				'description' => esc_html__( 'Cost of the one packaging. This cost will be added to the final amount of delivery.', 'russian-post-and-ems-for-woocommerce' ),
				'type'        => 'number',
				'default'     => 0,
			),

			'fixedvalue_disable' => array(
				'title'       => esc_html__( 'Min. cost of order', 'russian-post-and-ems-for-woocommerce' ),
				'description' => esc_html__( 'Disable this method if the cost of the order is less than inputted value. Leave this field empty to allow any order cost.', 'russian-post-and-ems-for-woocommerce' ),
				'type'        => 'number',
			),

			'overweight_disable' => array(
				'title'       => esc_html__( 'Do not allow overweight', 'russian-post-and-ems-for-woocommerce' ),
				'type'        => 'checkbox',
				'label'       => esc_html__( 'Disable this method in case of overweight.', 'russian-post-and-ems-for-woocommerce' ),
				'description' => esc_html__( 'Hide this method if package weight is heavier than the allowed weight for a chosen type of departure. By default, if there is overweight the package will be split into two or more packages.', 'russian-post-and-ems-for-woocommerce' ),
				'default'     => 'no',
			),

			'time' => array(
				'title'       => esc_html__( 'Delivery Time', 'russian-post-and-ems-for-woocommerce' ),
				'type'        => 'checkbox',
				'label'       => esc_html__( 'Show time of delivery.', 'russian-post-and-ems-for-woocommerce' ),
				'description' => esc_html__( 'Displayed next to the title. For international shipments, it works only for EMS - international.', 'russian-post-and-ems-for-woocommerce' ),
				'default'     => 'no',
			),
		);

		$this->title              = $this->get_option( 'title' );
		$this->from               = $this->get_option( 'from' );
		$this->addcost            = $this->get_option( 'addcost' );
		$this->type               = $this->get_option( 'type' );
		$this->addpackweight      = $this->get_option( 'addpackweight' );
		$this->addpackcost        = $this->get_option( 'addpackcost' );
		$this->fixedpackvalue     = $this->get_option( 'fixedpackvalue' );
		$this->overweight_disable = $this->get_option( 'overweight_disable' );
		$this->fixedvalue_disable = $this->get_option( 'fixedvalue_disable' );
		$this->time               = $this->get_option( 'time' );

		add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );

	}

	// check if curl is on
	public function _isCurl() {
		return function_exists( 'curl_version' );
	}

	/**
	 * calculate_shipping function.
	 *
	 * @param array $package (default: array())
	 */
	public function calculate_shipping( $package = array() ) {

		$from_code = $this->from;

		// origin postcode must be set
		if ( $from_code == '' ) {
			return;
		}

		global $woocommerce;

		$plugin_sw  = 'rpaefw_post_calc_1.2';
		$to_city    = '';
		$to_country = 'RU';

		$country_code = $package['destination']['country'];
		$postal_code  = wc_format_postcode( $package['destination']['postcode'], $country_code ? $country_code : 'RU' );
		$city         = $package['destination']['city'];

		// if country field is not empty
		if ( $country_code != '' ) {
			$to_country = $country_code;
		}

		// validate postcode
		if ( $postal_code != '' && ( $to_country == 'RU' && ! preg_match( '/^[1-9][0-9]{5}$/i', $postal_code ) ) ) {
			return;
		}

		// if postal code is empty take city
		if ( $postal_code != '' ) {
			// check if post code in a russian post base
			if ( $to_country == 'RU' ) {
				$validated_post_code = $this->get_default_ops( $postal_code );
				if ( ! $validated_post_code ) {
					return;
				}
			}
			$to_city = $postal_code;
		} else {
			// check if city in a russian post base
			if ( $to_country == 'RU' ) {
				$validated_city = $this->get_default_ops( $city );
				if ( ! $validated_city ) {
					return;
				}
			}
			$to_city = $city;
		}

		$admin_email = get_option( 'admin_email' );
		$site_url    = get_option( 'siteurl' );
		$site_url    = implode( '/', array_slice( explode( '/', preg_replace( '/https?:\/\/|www./', '', $site_url ) ), 0, 1 ) );

		$packweight = $this->addpackweight;

		// get weight of the cart
		// normalise weights, unify to g
		$weight = $woocommerce->cart->cart_contents_weight;
		$weight = wc_get_weight( $weight, 'g' );

		// plus pack weight
		$weight = $weight + $packweight;

		// get total value
		$totalval = $package['contents_cost'];

		if ( ! $to_city || $to_city == '' ) {
			return;
		}

		// additional cost
		$addcost     = intval( $this->addcost );
		$addpackcost = intval( $this->addpackcost );

		// get type of delivery
		$type = $this->type;

		// max weight of type 2kg
		$banderol = array(
			'ПростаяБандероль',
			'ЗаказнаяБандероль',
			'ЗаказнаяБандероль1Класс',
			'ЦеннаяБандероль',
			'ЦеннаяБандероль1Класс'
		);

		// max weight of intl type 2kg
		$paket_intl = array(
			'МждМелкийПакет',
			'МждМелкийПакетАвиа',
			'МждМелкийПакетЗаказной',
			'МждМелкийПакетАвиаЗаказной'
		);

		// max weight of intl type 5kg
		$banderol_intl = array(
			'МждБандероль',
			'МждБандерольАвиа',
			'МждБандерольЗаказная',
			'МждБандерольАвиаЗаказная'
		);

		// max weight of intl type 14.5kg
		$meshok_intl = array( 'МждМешокМ', 'МждМешокМАвиа', 'МждМешокМЗаказной', 'МждМешокМАвиаЗаказной' );
		// max weight of type 20kg
		$posilka = array( 'ПростаяПосылка', 'ЦеннаяПосылка', 'Посылка1Класс' );
		// max weight of type 31.5kg
		$emstype = array( 'EMS_МждДокументы', 'EMS_МждТовары' );

		// by default only one shipping
		$kolotprav = 1;

		// check if order exceeds limited of post
		if ( in_array( $type, $banderol ) && $weight >= '2000' ) {
			$kolotprav = intval( $weight / 2000 + 1 );
		} elseif ( in_array( $type, $paket_intl ) && $weight >= '2000' ) {
			$kolotprav = intval( $weight / 2000 + 1 );
		} elseif ( in_array( $type, $banderol_intl ) && $weight >= '5000' ) {
			$kolotprav = intval( $weight / 5000 + 1 );
		} elseif ( in_array( $type, $meshok_intl ) && $weight >= '14500' ) {
			$kolotprav = intval( $weight / 14500 + 1 );
		} elseif ( in_array( $type, $posilka ) && $weight >= '20000' ) {
			$kolotprav = intval( $weight / 20000 + 1 );
		} elseif ( in_array( $type, $emstype ) && $weight >= '31500' ) {
			$kolotprav = intval( $weight / 31500 + 1 );
		}

		// count new weight
		if ( $kolotprav > 1 ) {
			// if overweight is not allowed
			if ( $this->overweight_disable != 'no' ) {
				return;
			}

			$weight = $weight + ( $packweight * $kolotprav ) - $packweight;

		}

		// if cost is less than provided in options
		if ( $this->fixedvalue_disable != '' && $this->fixedvalue_disable > 0 ) {
			if ( $totalval < $this->fixedvalue_disable ) {
				return;
			}
		}

		// check weight and set minimum value for Russian Post if it is less than required
		$weight = $weight < 100 ? 100 : $weight;

		// fixed value
		$fixedvalue = intval( $this->fixedpackvalue );
		if ( $fixedvalue != '' && $totalval > $fixedvalue ) {
			$totalval = $fixedvalue;
		}

		// data
		$From      = $from_code;
		$To        = $to_city;
		$Weight    = $weight;
		$Valuation = $totalval;
		$Country   = $to_country;
		$Site      = $site_url;
		$Email     = $admin_email;

		// a query with all the variables
		$query = "f=" . rawurlencode( $From );
		$query .= "&t=" . rawurlencode( $To );
		$query .= "&w=$Weight&v=$Valuation&c=$Country&o=php&cs=utf-8";
		$query .= "&st=$Site&ml=$Email";
		$query .= "&sw=" . $plugin_sw;
		$query .= "&r=100"; // round

		// the request URL
		$request = "http://api.postcalc.ru/?$query";

		// a query option
		$arr_options = array(
			'http' =>
				array( 'header' => 'Accept-Encoding: gzip', 'timeout' => 5, 'user_agent' => phpversion() )
		);

		// get a cache response
		$arr_response = get_transient( $request );

		if ( false === $arr_response ) {
			$response = $this->connect_api_server( $request, $arr_options );

			if ( ! $response ) {
				return;
			}

			$arr_response = $this->unpack_answer( $response );
			// cache a response
			set_transient( $request, $arr_response, DAY_IN_SECONDS );
		}

		if ( isset( $arr_response['Status'] ) && $arr_response['Status'] != 'OK' ) {
			return;
		}

		// create new type based on ПростаяПосылка
		if ( $type == 'ПростаяПосылка' ) {
			$new_type = 'ЦеннаяПосылка';
		} else {
			$new_type = $type;
		}

		if ( ! isset( $arr_response['Отправления'][ $new_type ]['Тариф'] ) || $arr_response['Отправления'][ $new_type ]['Тариф'] == '0' ) {
			return;
		}

		$finalshippingcost = $arr_response['Отправления'][ $new_type ]['Доставка'];

		// if simple parcel take only tariff cost
		if ( $type == 'ПростаяПосылка' ) {
			$finalshippingcost = $arr_response['Отправления'][ $new_type ]['Тариф'];
		}

		// show time delivery if yes
		$time         = '';
		$deliverytime = isset( $arr_response['Отправления'][ $new_type ]['СрокДоставки'] ) ? $arr_response['Отправления'][ $new_type ]['СрокДоставки'] : false;

		if ( $this->time === 'yes' && $deliverytime ) {
			if ( strpos( $deliverytime, "-" ) !== false ) {
				$deliverytimeday = explode( "-", $deliverytime );
				$deliverytime    = $deliverytimeday[1];
			}
			$time = ' (' . sprintf( _n( '%s day', '%s days', $deliverytime, 'russian-post-and-ems-for-woocommerce' ), number_format_i18n( $deliverytime ) ) . ')';
		}

		// shipping cost + packages cost + additional cost
		$cost = intval( $finalshippingcost + $addcost + $addpackcost * $kolotprav );

		// check if default currency is different from RUB
		$currency = get_option( 'woocommerce_currency' );

		if ( $currency != 'RUB' ) {
			$cost = $this->get_currency_value( $currency, $cost );
		}

		// do not apply if no cost is set
		if ( ! $cost ) {
			return;
		}

		$this->add_rate( array(
			'id'    => $this->get_rate_id(),
			'label' => $this->title . $time,
			'cost'  => $cost,
		) );
	}

	/**
	 * Get currency value based on CBR rate
	 *
	 * @param $currency
	 * @param $cost
	 *
	 * @return int
	 */
	public function get_currency_value( $currency, $cost ) {
		// check if third party plugins are installed
		if ( in_array( 'woocommerce-currency-switcher/index.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
			$woocs = get_option('woocs');

			if ($woocs && is_array($woocs) && isset($woocs['RUB'])) {
				return round( 1 / $woocs['RUB']['rate'] * $cost );
			}
		}

		$rates = get_transient( 'rpaefw_post_calc_currency_rates' );

		if ( $rates === false ) {
			// try to get rates from CBR
			$new_rates = wp_remote_get( 'https://www.cbr.ru/scripts/XML_daily_eng.asp' );

			// if something wrong with request return false
			if ( is_array( $new_rates ) ) {
				$rates = $new_rates['body'];
			} else {
				return 0;
			}

			set_transient( 'rpaefw_post_calc_currency_rates', $rates );
		}

		$rates = simplexml_load_string( $rates );

		// update rates if it is outdated and today is not weekend
		if ( $rates['Date'] != date( 'd.m.Y' ) && date( 'w' ) < 5 ) {
			$new_rates = wp_remote_get( 'https://www.cbr.ru/scripts/XML_daily_eng.asp' );

			// if something wrong with request we will use old rates
			if ( is_array( $new_rates ) ) {
				$rates = $new_rates['body'];
				set_transient( 'rpaefw_post_calc_currency_rates', $rates );

				// parce new rates
				$rates = simplexml_load_string( $rates );
			}
		}

		$valute_obj = null;

		// find obj for provided currency
		foreach ( $rates->Valute as $valute ) {
			if ( $valute->CharCode == $currency ) {
				$valute_obj = $valute;
				break;
			}
		}

		// if no match is found return false
		if ( ! $valute_obj ) {
			return 0;
		}

		return round( 1 / str_replace( ',', '.', $valute_obj->Value ) * $cost );
	}

	/**
	 * Try to find destination based on provided type of address
	 *
	 * @param $to
	 *
	 * @return bool
	 */
	public function get_default_ops( $to ) {
		if ( $to == '000000' ) {
			return false;
		}

		// in case fopen is not working
		if ( ! ini_get( 'allow_url_fopen' ) ) {
			return true;
		}

		if ( preg_match( '/^[1-9][0-9]{5}$/', $to ) ) {
			// if it is a post code
			$isPindex = true;
			$arr      = $this->arr_from_txt( 'postcalc_light_post_indexes.txt', $to );
		} else {
			// any other symbols
			$isPindex = false;
			$arr      = $this->arr_from_txt( 'postcalc_light_cities.txt', $to );
		}

		// in case of no result from index files
		if ( $arr === false ) {
			return true;
		}

		// look for matches
		if ( isset( $arr[ $to ] ) ) {
			return ( $isPindex ) ? $to : $arr[ $to ];
		}

		return false;
	}

	/**
	 * Try to find destination based on provided type of address
	 *
	 * @param $src_txt
	 * @param $search
	 * @param $limit
	 *
	 * @return bool, array
	 */
	public function arr_from_txt( $src_txt, $search = '', $limit = 0 ) {
		$arr       = array();
		$config_cs = 'utf-8';

		$contex_options = array(
			"ssl" => array(
				"verify_peer"      => false,
				"verify_peer_name" => false,
			),
		);

		$context = stream_context_create( $contex_options );

		// source - index file
		$src_idx = basename( $src_txt, 'txt' ) . 'idx';
		$src_txt = plugin_dir_url( __FILE__ ) . 'post-calc/' . $src_txt;
		$src_idx = plugin_dir_url( __FILE__ ) . 'post-calc/' . $src_idx;
		$search  = mb_convert_case( $search, MB_CASE_LOWER, $config_cs );

		// deep looking if file does not exists or no search is provided
		if ( ! file_exists( $src_idx ) || $search == '' ) {
			if ( is_ssl() ) {
				$fp = fopen( $src_txt, 'r', false, $context );
			} else {
				$fp = fopen( $src_txt, 'r' );
			}
			if ( ! $fp ) {
				return false;
			}
			$counter = 0;
			while ( ( $line = fgets( $fp ) ) !== false ) {
				list( $key, $value ) = explode( "\t", $line );
				if ( $search == '' ||
				     ( $search != '' && mb_stripos( $key, $search, 0, $config_cs ) === 0 )
				) {
					$arr[ $key ] = trim( $value );
					if ( $limit > 0 && ++ $counter >= $limit ) {
						break;
					}
				}
			}
			fclose( $fp );

			return $arr;
		}

		// index file
		if ( is_ssl() ) {
			$string_idx = file_get_contents( $src_idx, false, $context );
		} else {
			$string_idx = file_get_contents( $src_idx );
		}

		if ( ! $string_idx ) {
			return false;
		}

		// look for matches
		$pos     = mb_strpos(
			$string_idx,
			// take first two symbols
			"\n" . mb_substr( $search, 0, 2, $config_cs ),
			0,
			$config_cs
		);
		$idx_len = mb_strlen( $string_idx, $config_cs );

		// end of matching string
		$pos_line_end = mb_strpos( $string_idx, "\n", $pos + 1, $config_cs );
		$s            = mb_substr( $string_idx, $pos + 1, $pos_line_end - $pos - 1, $config_cs );

		// get from original file
		list( $tmp, $offset ) = explode( "\t", $s );

		if ( $idx_len == $pos_line_end + 1 ) {
			// if it is a last string in file take all fragments until the end
			// take any big number
			$len = 1000000;
		} else {
			$pos          = $pos_line_end + 1;
			$pos_line_end = mb_strpos( $string_idx, "\n", $pos + 1, $config_cs );
			$s            = mb_substr( $string_idx, $pos + 1, $pos_line_end - $pos, $config_cs );
			list( $tmp, $offset2 ) = explode( "\t", $s );
			$len = $offset2 - $offset;
		}

		if ( is_ssl() ) {
			$fp = fopen( $src_txt, 'r', false, $context );
		} else {
			$fp = fopen( $src_txt, 'r' );
		}

		if ( ! $fp ) {
			return false;
		}

		fseek( $fp, $offset );
		$chunk = fread( $fp, $len );
		fclose( $fp );

		// create an array
		$arr_tmp = explode( "\n", trim( $chunk ) );
		$counter = 0;

		foreach ( $arr_tmp as $no => $line ) {
			list( $key, $value ) = explode( "\t", $line );
			if ( $search == '' || ( $search != '' && mb_stripos( $key, $search, 0, $config_cs ) === 0 ) ) {
				$arr[ $key ] = trim( $value );
				if ( $limit > 0 && ++ $counter >= $limit ) {
					break;
				}
			}

		}

		return $arr;
	}

	/**
	 * Connecting to the server
	 *
	 * @param $request
	 * @param $arr_options
	 *
	 * @return array
	 */
	public function connect_api_server( $request, $arr_options ) {
		if ( $this->_isCurl() ) {
			$curl = curl_init();

			curl_setopt_array( $curl, array(
				CURLOPT_URL            => $request,
				CURLOPT_RETURNTRANSFER => 1,
				CURLOPT_CONNECTTIMEOUT => 5,
				CURLOPT_HTTPHEADER     => array( 'Connection: close', 'Accept-Encoding: gzip' ),
				CURLOPT_USERAGENT      => phpversion()
			) );

			$response = curl_exec( $curl );
			curl_close( $curl );

		} else {
			$response = @file_get_contents( $request, false, stream_context_create( $arr_options ) );
		}

		return $response;
	}

	/**
	 * Unpack the answer from server api
	 *
	 * @param $response
	 *
	 * @return array
	 */
	public function unpack_answer( $response ) {
		// unpack the answer
		if ( substr( $response, 0, 3 ) == "\x1f\x8b\x08" ) {
			$response = gzinflate( substr( $response, 10, - 8 ) );
			if ( $response === false ) {
				// in some hosting the result can be cut after unziping
				$response = gzinflate( substr( $response, 10, - 1 ) );
			}
		}

		// switch into a PHP array
		return unserialize( $response );
	}
}
